@@ -33,7 +33,7 @@
 
 #define SUBFOLDER_DIR_NAME     "subfolders"
 #define SUBFOLDER_DIR_NAME_LEN 10
-#define d(x) (x) 
+#define d(x)
 
 
 //static CamelStoreClass *parent_class = NULL;
@@ -289,6 +289,29 @@ get_name (CamelService *service, gboolean brief)
 
 #define EXCHANGE_STOREINFO_VERSION 1
 
+/* SURF : Picked this from gal/util/e-util.c */
+/* This only makes a filename safe for usage as a filename.  
+	It still may have shell meta-characters in it. */
+static void
+e_filename_make_safe (gchar *string)
+{
+	gchar *p, *ts;
+	gunichar c;
+	
+	g_return_if_fail (string != NULL);
+	p = string;
+
+	while(p && *p) {
+		c = g_utf8_get_char (p);
+		ts = p;
+		p = g_utf8_next_char (p);
+		if (!g_unichar_isprint(c) || ( c < 0xff && strchr (" /'\"`&();|<>$%{}!", c&0xff ))) {
+			while (ts<p) 	
+				*ts++ = '_';
+		}
+	}
+}
+
 static gboolean
 exchange_connect (CamelService *service, CamelException *ex)
 {
@@ -403,7 +426,7 @@ exchange_folder_subscribed (CamelStore *store, const char *folder_name)
 	guint32 is_subscribed;
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
-		return;
+		return FALSE;
 	}
 
 	if (!camel_stub_send (exch->stub, NULL, CAMEL_STUB_CMD_IS_SUBSCRIBED_FOLDER,
@@ -600,6 +623,7 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 	g_array_free (folder_flags, TRUE);
 
 	info = camel_folder_info_build (folders, top, '/', TRUE);
+	
 	if (info)
 		info = postprocess_tree (info);
 	g_ptr_array_free (folders, TRUE);
@@ -709,6 +733,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 	{
 		CamelExchangeFolder *folder;
 		char *folder_name, *uid;
+		CamelMessageInfo *info;
 
 		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1 ||
 		    camel_stub_marshal_decode_string (stub->status, &uid) == -1)
@@ -717,8 +742,10 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
 		g_mutex_unlock (exch->folders_lock);
-		if (folder)
+		if (folder && (info = camel_folder_summary_uid (((CamelFolder *)folder)->summary, uid))) {
+			camel_message_info_free (info);
 			camel_exchange_folder_remove_message (folder, uid);
+		}
 
 		g_free (folder_name);
 		g_free (uid);
@@ -839,7 +866,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		info = make_folder_info (exch, name, uri, -1, 0);
 		info->flags |= CAMEL_FOLDER_NOCHILDREN;
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "folder_created", info);
+					    "folder_subscribed", info);
 		camel_folder_info_free (info);
 		break;
 	}
@@ -847,15 +874,25 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 	case CAMEL_STUB_RETVAL_FOLDER_DELETED:
 	{
 		CamelFolderInfo *info;
+		CamelFolder *folder;
 		char *name, *uri;
 
 		if (camel_stub_marshal_decode_string (stub->status, &name) == -1 ||
 		    camel_stub_marshal_decode_string (stub->status, &uri) == -1)
 			break;
-
+		
 		info = make_folder_info (exch, name, uri, -1, 0);
+
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, info->full_name);
+		if (folder) {
+			g_hash_table_remove (exch->folders, info->full_name);
+			camel_object_unref (CAMEL_OBJECT (folder));
+		}
+		g_mutex_unlock (exch->folders_lock);
+
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "folder_deleted", info);
+					    "folder_unsubscribed", info);
 		camel_folder_info_free (info);
 		break;
 	}
@@ -863,6 +900,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 	case CAMEL_STUB_RETVAL_FOLDER_RENAMED:
 	{
 		char *new_name, *new_uri;
+		CamelFolder *folder;
 		CamelRenameInfo reninfo;
 
 		if (camel_stub_marshal_decode_folder (stub->status, &reninfo.old_base) == -1 ||
@@ -871,6 +909,15 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 			break;
 
 		reninfo.new = make_folder_info (exch, new_name, new_uri, -1, 0);
+		
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, reninfo.old_base);
+		if (folder) {
+			g_hash_table_remove (exch->folders, reninfo.old_base);
+			camel_object_unref (CAMEL_OBJECT (folder));
+		}
+		g_mutex_unlock (exch->folders_lock);
+		
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
 					    "folder_renamed", &reninfo);
 		camel_folder_info_free (reninfo.new);
