@@ -49,8 +49,6 @@ static gboolean idle_do_interactive (gpointer user_data);
 struct ExchangeComponentPrivate {
 	GdkNativeWindow xid;
 
-	//EFolderTypeRegistry *folder_type_registry;
-
 	ExchangeConfigListener *config_listener;
 	ExchangeOfflineListener *offline_listener;
 	GSList *accounts;
@@ -77,13 +75,6 @@ dispose (GObject *object)
 	ExchangeComponentPrivate *priv = EXCHANGE_COMPONENT (object)->priv;
 	GSList *p;
 
-#if 0
-	if (priv->folder_type_registry) {
-		g_object_unref (priv->folder_type_registry);
-		priv->folder_type_registry = NULL;
-	}
-#endif
-
 	if (priv->config_listener) {
 		g_object_unref (priv->config_listener);
 		priv->config_listener = NULL;
@@ -117,7 +108,6 @@ finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-#if 0
 static void
 impl_createControls (PortableServer_Servant servant,
 		     Bonobo_Control *sidebar_control,
@@ -125,36 +115,10 @@ impl_createControls (PortableServer_Servant servant,
 		     Bonobo_Control *statusbar_control,
 		     CORBA_Environment *ev)
 {
-	ExchangeComponent *component = EXCHANGE_COMPONENT (bonobo_object_from_servant (servant));
-	ExchangeComponentPrivate *priv = component->priv;
-	XCBackendView *view;
-	BonoboControl *control = NULL;
-
 	d(printf("createControls...\n"));
 
-     *sidebar_control = *view_control = *statusbar_control = NULL;
-	//bonobo_exception_general_error_set (ev, NULL, "We do not want to have a component\n");
-	bonobo_exception_set (ev, ex_Bonobo_Storage_NotSupported);
-	return;
-
-	view = xc_backend_view_new (priv->config_listener,
-				    priv->folder_type_registry);
-	if (view)
-		priv->views = g_slist_append (priv->views, view);
-
-	control = xc_backend_view_get_sidebar (view);
-	*sidebar_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
-
-	control = xc_backend_view_get_statusbar (view);
-	*statusbar_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
-
-	control = xc_backend_view_get_view (view);
-	*view_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
+	*sidebar_control = *view_control = *statusbar_control = NULL;
 }
-#endif
 
 static void
 impl_upgradeFromVersion (PortableServer_Servant servant,
@@ -319,45 +283,6 @@ config_listener_account_removed (ExchangeConfigListener *config_listener,
 	}
 }
 
-
-static struct {
-	char *type, *display_name, *icon;
-} folder_types[] = {
-	{ "mail", N_("Mail"), "stock_folder" },
-	{ "mail/public", NULL, "stock_folder" },
-
-	{ "contacts", N_("Contacts"), "stock_contact" },
-	{ "contacts/public", NULL, "stock_contact" },
-	{ "contacts/ldap", NULL, "stock_contact" },
-
-	{ "tasks", N_("Tasks"), "stock_todo" },
-	{ "tasks/public", NULL, "stock_todo" },
-
-	{ "calendar", N_("Calendar"), "stock_calendar" },
-	{ "calendar/public", NULL, "stock_calendar" }
-};
-static int n_folder_types = G_N_ELEMENTS (folder_types);
-
-#if 0
-static void
-setup_folder_type_registry (ExchangeComponent *component)
-{
-	ExchangeComponentPrivate *priv = component->priv;
-	int i;
-
-	priv->folder_type_registry = e_folder_type_registry_new ();
-	for (i = 0; i < n_folder_types; i++) {
-		e_folder_type_registry_register_type (priv->folder_type_registry,
-						      folder_types[i].type,
-						      folder_types[i].icon,
-						      _(folder_types[i].display_name),
-						      NULL,
-						      folder_types[i].display_name != NULL,
-						      0, NULL);
-	}
-}
-#endif
-
 static void
 exchange_component_class_init (ExchangeComponentClass *klass)
 {
@@ -369,7 +294,7 @@ exchange_component_class_init (ExchangeComponentClass *klass)
 	object_class->dispose = dispose;
 	object_class->finalize = finalize;
 
-	//epv->createControls     = impl_createControls;
+	epv->createControls     = impl_createControls;
 	epv->upgradeFromVersion = impl_upgradeFromVersion;
 	epv->requestQuit        = impl_requestQuit;
 	epv->quit               = impl_quit;
@@ -384,8 +309,6 @@ exchange_component_init (ExchangeComponent *component)
 
 	priv = component->priv = g_new0 (ExchangeComponentPrivate, 1);
 
-	//setup_folder_type_registry (component);
-
        	priv->config_listener = exchange_config_listener_new ();
 	g_signal_connect (priv->config_listener, "exchange_account_created",
 			  G_CALLBACK (config_listener_account_created),
@@ -432,8 +355,13 @@ exchange_component_get_account_for_uri (ExchangeComponent *component,
 		if (!uri)
 			return baccount->account;
 
-		if (exchange_account_get_folder (baccount->account, uri))
+		if (exchange_account_get_folder (baccount->account, uri)) {
 			return baccount->account;
+		} else {
+			exchange_account_rescan_tree (baccount->account);
+			if (exchange_account_get_folder (baccount->account, uri))
+				return baccount->account;
+		}
 		/* FIXME : Handle multiple accounts */
 	}
 	return NULL;
@@ -452,3 +380,10 @@ exchange_component_is_interactive (ExchangeComponent *component)
 {
 	return component->priv->xid != 0;
 }
+
+ExchangeOfflineListener *
+exchange_component_get_offline_listener (ExchangeComponent *component)
+{
+	return component->priv->offline_listener;
+}
+
